@@ -223,19 +223,39 @@ class DebianSourcePackage(unperishlib.Operation):
 
     """Create a Debian source package, if one doesn't already exist."""
     
-    name = "debian-source-package"
+    name = "dsc"
 
     required_options = ["debian_source_name", "debian_upstream_version",
                         "debian_version", "debian_orig_tar_gz", 
                         "upstream_tarball", "build_area_exists", 
                         "debian_source_tree"]
 
-    provided_options = ["debian_dsc"]
+    provided_options = ["debian_dsc", "debian_source_changes"]
     
     def add_options(self, parser):
         parser.add_option("--debian-dsc", metavar="FILE",
                           help="Use FILE as the Debian .dsc file. "
                                "Default is computed from debian/changelog.")
+        parser.add_option("--debian-source-changes", metavar="FILE",
+                          help="Use FILE as the Debian _source.changes file. "
+                               "Default is what dpkg-buildpackage creates.")
+
+        parser.add_public_option("--source-buildpackage-options",
+                                 metavar="OPTIONS",
+                                 default="",
+                                 help="""\
+Give OPTIONS to dpkg-buildpackage when creating a source package.
+Multiple options can be given in OPTIONS by separating them with spaces.
+
+Note that this only applies to building of source packages, not
+binary packages. See --debbuildopts for that.
+""")
+
+    def source_changes(self, options):
+        """Return full pathname to the source.changes file."""
+        basename = "%s_%s_source.changes" % (options.debian_source_name,
+                                             options.debian_version)
+        return os.path.join(options.build_area, basename)
 
     def do_it(self, options):
         if options.debian_dsc is None:
@@ -249,11 +269,15 @@ class DebianSourcePackage(unperishlib.Operation):
                 origtgz = options.debian_orig_tar_gz
                 if os.path.dirname(origtgz) == options.build_area:
                     origtgz = os.path.basename(origtgz)
-            unperishlib.run(["dpkg-source", "-b", 
-                             options.debian_source_tree,
-                             origtgz],
-                            cwd=options.build_area)
 
+            args = ["dpkg-buildpackage", "-S", "-us", "-uc", "-rfakeroot"]
+            args += options.source_buildpackage_options.split()
+
+            unperishlib.run(args, cwd=options.debian_source_tree)
+
+        if not options.debian_source_changes:
+            options.debian_source_changes = self.source_changes(options)
+                                                         
 
 class DebianArchitecture(unperishlib.Operation):
 
@@ -282,11 +306,11 @@ class DebianBinaryPackages(unperishlib.Operation): #pragma: no cover
 
     """Create Debian binary packages."""
     
-    name = "debian-binary-packages"
+    name = "deb"
     
     required_options = ["build_area", "debian_dsc", "debian_architecture"]
     
-    provided_options = ["debian_changes"]
+    provided_options = ["debian_binary_changes"]
     
     def add_options(self, parser):
         parser.add_public_option("--pbuilder-basetgz", metavar="FILE",
@@ -294,27 +318,85 @@ class DebianBinaryPackages(unperishlib.Operation): #pragma: no cover
 Use FILE as the pbuilder basetgz file.
 Default is to use whatever pbuilder has as its default.""")
 
-        parser.add_option("--debian-changes", metavar="FILE",
-                          help="Use FILE as the Debian .changes files. "
+        parser.add_public_option("--debbuildopts", 
+                                 metavar="OPTIONS",
+                                 help="""\
+Give OPTIONS to pbuilder as the value of pbuilder's --debbuildopts
+option. Pbuilder will then parse the string and give the options in
+the string to dpkg-buildpackage when building the package. Multiple
+options can be given in OPTIONS by separating them with spaces.
+
+Note that this only applies to building of binary packages, not
+source packages. See --source-buildpackage-options for that.
+""")
+
+        parser.add_option("--debian-binary-changes", metavar="FILE",
+                          help="Use FILE as the Debian binary .changes file. "
                                "The default is computed from "
                                "debian/changelog.")
 
     def do_it(self, options):
-        if options.debian_changes is None:
-            options.debian_changes = os.path.join(options.build_area,
-                                              "%s_%s_%s.changes" % 
-                                              (options.debian_source_name,
-                                               options.debian_version,
-                                               options.debian_architecture))
+        logging.debug("In debian-binary-packages")
+        if options.debian_binary_changes is None:
+            options.debian_binary_changes = \
+                os.path.join(options.build_area,
+                             "%s_%s_%s.changes" % 
+                             (options.debian_source_name,
+                              options.debian_version,
+                              options.debian_architecture))
+            logging.debug("Set debian_binary_changes to %s" % 
+                          repr(options.debian_binary_changes))
 
-        if not os.path.exists(options.debian_changes):
+        if not os.path.exists(options.debian_binary_changes):
             args = ["sudo", "-p", "Password (for sudo to run pbuilder): ",
-                    "pbuilder", "build", "--buildresult", options.build_area]
+                    "pbuilder", "--build", 
+                    "--buildresult", options.build_area]
             if options.pbuilder_basetgz:
                 args.append("--basetgz")
                 args.append(options.pbuilder_basetgz)
+
+            debbuildopts = "-b"
+            if options.debbuildopts:
+                debbuildopts += options.debbuildopts
+            args.append("--debbuildopts")
+            args.append(debbuildopts)
+
             args.append(options.debian_dsc)
             unperishlib.run(args, cwd=options.build_area)
+        else:
+            logging.debug(".changes file already exists, nothing to do")
+
+
+class DebianChanges(unperishlib.Operation):
+
+    """Provide a Debian .changes file."""
+    
+    name = "debian-changes"
+    
+    provided_options = ["debian_changes"]
+    
+    def add_options(self, parser):
+        parser.add_option("--debian-changes", metavar="FILE",
+                          help="Use FILE as the Debian .changes file, "
+                               "either the source or binary one.")
+
+    def do_it(self, options):
+        if not options.debian_changes:
+            if options.debian_binary_changes:
+                logging.debug("Setting debian_changes to "
+                              "debian_binary_changes: %s" % 
+                              options.debian_binary_changes)
+                options.debian_changes = options.debian_binary_changes
+            elif options.debian_source_changes:
+                logging.debug("Setting debian_changes to "
+                              "debian_source_changes: %s" % 
+                              options.debian_source_changes)
+                options.debian_changes = options.debian_source_changes
+            else:
+                logging.debug("Neither source nor binary .changes set. "
+                              "Can't set debian_changes.")
+        else:
+            logging.debug("debian_changes already set")
 
 
 class DebsignChanges(unperishlib.Operation): #pragma: no cover
@@ -325,7 +407,7 @@ class DebsignChanges(unperishlib.Operation): #pragma: no cover
 
     """Run debsign on a .changes file."""
     
-    name = "debsign-changes"
+    name = "debsign"
     
     required_options = ["debian_changes", "directory"]
     
@@ -353,24 +435,6 @@ class Lintian(unperishlib.Operation): #pragma: no cover
                                     for line in output.splitlines()))
 
 
-class Linda(unperishlib.Operation): #pragma: no cover
-
-    # We have no easy way of generating binary debs for unit testing, so
-    # this class is excluded from coverage metering.
-
-    """Check Debian packages with linda."""
-    
-    name = "linda"
-    
-    required_options = ["debian_changes"]
-
-    def do_it(self, options):
-        output = unperishlib.run(["linda", options.debian_changes])
-        if output:
-            logging.error("Linda found problems:\n%s" %
-                          "\n".join("  " + line 
-                                    for line in output.splitlines()))
-
 
 class Piuparts(unperishlib.Operation): #pragma: no cover
 
@@ -493,3 +557,35 @@ class UpdateAptRepository(unperishlib.Operation): #pragma: no cover
         unperishlib.run(["reprepro", "-b", options.apt_repository, 
                          "include", distro, options.debian_changes],
                          cwd=options.build_area)
+
+
+class Dput(unperishlib.Operation):
+
+    """Upload a Debian source or binary package with dput."""
+    
+    name = "dput"
+    
+    required_options = ["debian_changes", "dput_host", "debian_source_name",
+                        "debian_version", "debian_architecture"]
+    
+    def add_options(self, parser):
+        parser.add_public_option("--dput-host", metavar="HOST",
+                                 help="Upload to HOST when using dput.")
+
+    def upload_name(self, options):
+        """Return full pathname to the .upload file."""
+        basename = "%s_%s_%s.upload" % (options.debian_source_name,
+                                        options.debian_version,
+                                        options.debian_architecture)
+        return os.path.join(options.build_area, basename)
+
+    def do_it(self, options): #pragma: no cover
+        upload = self.upload_name(options)
+        if os.path.exists(upload):
+            logging.debug("Upload file %s exists already" % upload)
+        else:
+            logging.debug("Uploading Debian package with dput to %s" %
+                          options.dput_host)
+            args = ["dput", options.dput_host, options.debian_changes]
+            logging.debug("dput: %s" % args)
+            unperishlib.run(args, cwd=options.build_area)
