@@ -33,13 +33,14 @@ class FakeOptions:
     """Fake an optparse Options object for unit tests."""
 
     def __init__(self, **kwargs):
-        self.project_name = None
-        self.project_version = None
         self.directory = "."
 
         for name, value in kwargs.iteritems():
             setattr(self, name, value)
 
+    def __getattr__(self, name):
+        return None
+
 
 class Stack:
 
@@ -381,7 +382,8 @@ class Application:
         """
         self._pm = PluginManager()
         if plugins is None:
-            plugins = self.default_plugin_dirs
+            plugins = [os.path.expanduser(dir) 
+                       for dir in self.default_plugin_dirs]
         self._pm.set_plugin_directories(plugins)
         self._cli = CommandLineParser()
         self._verbose_file = sys.stdout
@@ -530,6 +532,7 @@ def run(argv, cwd=None):
     spawns the sub-process).
     
     """
+    logging.debug("Executing (in %s): %s" % (cwd, repr(argv)))
     p = subprocess.Popen(argv, cwd=cwd, stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE)
     stdout, stderr = p.communicate()
